#!/usr/bin/env gosh
;; -*- coding: euc-jp -*-

(use text.html-lite)
(use text.tree)

(define *version* "0.2.0")
(define *last-update* "Tue Jan 22 2008")

(define-syntax def
  (syntax-rules (en ja procedure method)
	((_ en)
     '())
    ((_ ja)
	 '())
	((_ en (synopsis x y z ...) rest ...)
     (cons
      (def (synopsis x z ...))
      (def en rest ...)))
	((_ ja (synopsis x y z ...) rest ...)
     (cons
      (def (synopsis y z ...))
      (def ja rest ...)))
	((_ ((procedure (name arg ...) ...) (p ...) z ...))
     (list
      (html:h3 (html:span :class "type" "procedure") ": "
               (html:span :class "procedure" (html-escape-string (symbol->string 'name))) " "
               (cons (html:span :class "argument" 'arg) " ") ...)
      ...
      (map
       (lambda (x)
         (if (string? x)
             (html:p (html-escape-string x))
             (html:pre (html-escape-string (list-ref '(z ...) x)))))
       (list p ...))
      (html:hr)))
	((_ ((method (name arg ...) ...) (p ...) z ...))
	 (list
	  (html:h3 (html:span :class "type" "method") ": "
			   (html:span :class "method" (html-escape-string (symbol->string 'name))) " "
			   (cons (html:span :class "argument" (html-escape-string (x->string 'arg))) " ") ...)
	  ...
      (map
       (lambda (x)
         (if (string? x)
             (html:p (html-escape-string x))
             (html:pre (html-escape-string (list-ref '(z ...) x)))))
       (list p ...))
	  (html:hr)))
	((_ ((type name ...) (p ...) z ...))
	 (list
	  (html:h3 (html:span :class "type" 'type) ": "
			   (html:span :class 'type (html-escape-string (symbol->string 'name))))
	  ...
      (map
       (lambda (x)
         (if (string? x)
             (html:p (html-escape-string x))
             (html:pre (html-escape-string (list-ref '(z ...) x)))))
       (list p ...))
	  (html:hr)))))

(define-macro (api-libavcodec lang)
  `(def ,lang

	   ((class <avcodec-context>)
		("Each instance of the class has a pointer to its own AVCodecContext.")
		("ƥ󥹥󥹤 AVCodecContext ؤΥݥ󥿤ޤ"))

	   ((method (close-input-avcodec (c <avcodec-context>)))
		("Close the <avcodec-context> `c'. GC will do that finally even if you leave it open.")
		("<avcodec-context> `c' ĤޤΤޤޤˤƤ gc ǽŪĤޤ"))

	   ((procedure (avcodec-version))
		("Return the libavcodec's version, e.g. \"51.27.0\".")
		("libavcodec ΥС(: \"51.27.0\")ʸȤ֤ޤ"))

	   ((method (get-bit-rate (c <avcodec-context>)))
		("(For audio only) Return the bit rate of `c'.")
		("(ǥΤ) `c' Υӥåȥ졼Ȥ֤ޤ"))

	   ((method (get-sample-rate (c <avcodec-context>)))
		("(For audio only) Return the sample rate of `c'.")
		("(ǥΤ) `c' Υץ졼Ȥ֤ޤ"))

	   ((method (get-width (c <avcodec-context>)))
		("(For video only) Return the width of `c'.")
		("(ӥǥΤ) `c' β֤ޤ"))

	   ((method (get-height (c <avcodec-context>)))
		("(For video only) Return the height of `c'.")
		("(ӥǥΤ) `c' ν֤ޤ"))

	   ((method (get-codec-name (c <avcodec-context>)))
		("Return its codec name as a string, or #f if failed.")
		("ǥå̾ʸ֤ޤ̾Ǥʤ #f ֤ޤ"))
	   ))

(define-macro (api-libavformat lang)
  `(def ,lang

	   ((class <avformat-context>)
		("Each instance of the class has a pointer to its own AVFormatContext.")
		("ƥ󥹥󥹤 AVFormatContext ؤΥݥ󥿤ޤ"))

	   ((procedure (avformat-version))
		("Return the libavformat's version, e.g. \"51.6.0\".")
		("libavformat ΥС(: \"51.6.0\")ʸȤ֤ޤ"))

	   ((method (open-input-avformat (path <string>)))
		("Open and return a <avformat-context> of the av file of path `path', or #f if falied.")
		("`path' ˤե򿷤˳ <avformat-context> ֤ޤԤˤ #f ֤ޤ"))

	   ((method (close-input-avformat (c <avformat-context>)))
		("Close a <avformat-context> `c'. GC will do that after all if you do not care.")
		("`c' Ĥޤ줿 <avformat-context> 򤽤ΤޤޤˤƤ GC ˤäĤޤ"))

	   ((method (get-duration (c <avformat-context>)))
		("Return the number of seconds in duration of `c'")
		("`c' κ֤ñ̤֤ޤ"))

	   ((method (get-file-name (c <avformat-context>)))
		("Return the plain name of the media file of `c', or #f if failed.")
		("`c' ΥǥեΥե֤̾ޤե̾Ǥʤ #f ֤ޤ"))

	   ((procedure (call-with-input-avformat path proc))
		("Call `proc' with a <avformat-context>, which newly open the file of path `path', as a single argument, and return the value `proc' does in case of success."
		 "The <avformat-context> will be closed before the procedure returns with an error or without.")
		("`path' ˤե򿷤˳ <avformat-context> ͣΰȤ `proc' ƤӤޤˤ `proc' ֤֤ͤޤ"
		 "μ³ˤ <avformat-context> Ĥޤ"))
	   ))

(define-macro (api-libavdevice lang)
  `(def ,lang
        ((procedure (avdevice-version))
         ("Return the libavdevice's version, e.g. \"52.0.0\".")
         ("libavdevice ΥС(: \"52.0.0\")ʸȤ֤ޤ"))
        ))

(define-macro (api-ffmpeg lang)
  `(def ,lang
		((method (open-input-acodec (c <avformat-context>))
				 (open-input-vcodec (c <avformat-context>))
				 (open-input-avcodec (c <avformat-context>)))
		 ("The first form opens and returns a audio <avcodec-context> from `c'. The second one does a video <avcodec-context>. And the last simultaneously returns both values i.e. audio and video codec contexts in that order."
		  "Any of them returns #f if failed.")
		 ("ǽΥ᥽åɤ `c' 饪ǥ <avcodec-context> 򳫤֤ޤ2ܤƱ褦˥ӥǥ <avcodec-context> ֤ޤƺǸƱ˥ǥӥǥξͤ򤳤ν֤֤ޤ"
		  "ԤϤ #f ֤ޤ"))

		((method (get-frame-rate (fc <avformat-context>) (cc <avcodec-context>)))
		 ("Return two numbers: the numerator and denominator of the frame rate of video with `fc' and `cc'."
		  "Given `cc' must be a video context."
		  "Note: the mothod will be not idempotent (currently at least) since it may occur a side effect for `fc'.")
		 ("ե졼졼ȤʬҤʬ2Ĥο֤ͤޤ"
		  "Ϳ`cc' ϥӥǥ context ǤʤФʤޤ"
		  ": Υ᥽åɤ(ʤȤ⸽Ǥ)֤ƤӽФȰǤʤ⤷ޤ"))

		((procedure (call-with-input-avcodec fc proc))
		 ("Call `proc' with the following two arguments: an audio <avcodec-context> and video <avcodec-context>, which are newly opened from `fc'."
		  "Return the value `proc' does in case of success. The <avcodec-context>s will be closed before the procedure returns with an error or without.")
		 ("`proc' 򼡤2ĤΰȤȤ˸ƤӤޤ: `fc' 鳫(1)ǥ <avcodec-context> (2)ӥǥ <avcodec-context>"
		  "礳μ³ `proc' ͤᤷޤ줿 context ϼ³Ĥޤ"))
		))

(define (document-tree lang)
  (let ((title (if (eq? 'ja lang) "Gauche-ffmpeg ե󥹥ޥ˥奢" "Gauche-ffmpeg Reference Manual")))
	(html:html
	 (html:head
	  (if (eq? 'ja lang) (html:meta :http-equiv "Content-Type" :content "text/html; charset=UTF-8") '())
	  (html:title title))
	 (html:body
	  (html:h1 title)
	  (html:style
	   :type "text/css"
	   "<!-- \n"
	   "h2 { background-color:#dddddd; }\n"
	   "address { text-align: right; }\n"
	   ".type { font-size: medium; text-decoration: underline; }\n"
	   ".procedure { font-size: medium; font-weight: normal; }\n"
	   ".method { font-size: medium; font-weight: normal; }\n"
	   ".argument { font-size: small; font-style: oblique; font-weight: normal; }\n"
	   ".constant { font-size: medium; font-weight: normal; }\n"
	   ".variable { font-size: medium; font-weight: normal; }\n"
	   "#last_update { text-align: right; font-size: small; }\n"
	   "#project { text-align: right; }\n"
	   " -->")
	  (html:p "For version " *version*)
	  (html:p :id "last_update" "last update: " *last-update*)
	  (html:p :id "project" (html:a :href "http://www.fixedpoint.jp/gauche-ffmpeg/" "http://www.fixedpoint.jp/gauche-ffmpeg/"))
	  (if (eq? 'en lang)
		  (html:p (html:span :style "color:red;" "Warning:") " still unstable.")
		  (html:p (html:span :style "color:red;" "ٹ:") " ѹβǽޤ"))
	  (html:h2 "API for libavcodec")
	  (if (eq? 'en lang)
		  (api-libavcodec en)
		  (api-libavcodec ja))
	  (html:h2 "API for libavformat")
	  (if (eq? 'en lang)
		  (api-libavformat en)
		  (api-libavformat ja))
      (html:h2 "API for libavdevice")
      (if (eq? 'en lang)
          (api-libavdevice en)
          (api-libavdevice ja))
	  (html:h2 "API for ffmpeg")
	  (if (eq? 'en lang)
		  (api-ffmpeg en)
		  (api-ffmpeg ja))
	  (html:address "&copy; 2006,2007 Takeshi Abe")
	  ))))

(define (main args)
  (define (usage)
	(format (current-error-port) "usage: gosh reference.scm (en|ja)\n")
	(exit 1))
  (when (< (length args) 2)
	(usage))
  (write-tree (document-tree (string->symbol (cadr args))))
  0)
